SystemOrganization addCategory: #'GT-EventRecorder'!
SystemOrganization addCategory: 'GT-EventRecorder-Announcements'!
SystemOrganization addCategory: 'GT-EventRecorder-Core'!
SystemOrganization addCategory: 'GT-EventRecorder-Exceptions'!
SystemOrganization addCategory: 'GT-EventRecorder-Identifier'!
SystemOrganization addCategory: 'GT-EventRecorder-Privacy'!
SystemOrganization addCategory: 'GT-EventRecorder-Settings'!

Error subclass: #GTEventRecorderError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Exceptions'!

!GTEventRecorderError commentStamp: 'JurajKubelka 10/30/2015 12:16' prior: 0!
I am an abstract class for all errors related to GT-EventRecorder.!

GTEventRecorderError subclass: #GTEventInitializationError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Exceptions'!

!GTEventInitializationError commentStamp: 'JurajKubelka 10/30/2015 12:18' prior: 0!
I represent an error because of unitialized object.

Collaborators: GTEventUnpacking
!

GTEventRecorderError subclass: #GTEventUnpackingError
	instanceVariableNames: 'data errors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Exceptions'!

!GTEventUnpackingError commentStamp: 'JurajKubelka 10/30/2015 12:19' prior: 0!
I represent an error that appears when a data cannot be unpacked (materialized).

Collaborators: GTEventUnpacking
!

!GTEventUnpackingError class methodsFor: 'instance creation' stamp: 'JurajKubelka 10/30/2015 12:05'!
signal: aString data: aByteArray errors: aCollection 
	^ self new
			data: aByteArray;
			errors: aCollection;
			signal: aString;
			yourself! !

!GTEventUnpackingError methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 12:05'!
data
	^ data! !

!GTEventUnpackingError methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 12:05'!
data: anObject
	data := anObject! !

!GTEventUnpackingError methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 12:05'!
errors
	^ errors! !

!GTEventUnpackingError methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 12:05'!
errors: anObject
	errors := anObject! !

!GTEventUnpackingError methodsFor: 'private' stamp: 'JurajKubelka 11/1/2015 13:05'!
isResumable
	"Determine whether an exception is resumable."
	^true! !

Announcement subclass: #GTEventAnnouncement
	instanceVariableNames: 'dictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Announcements'!

!GTEventAnnouncement commentStamp: 'JurajKubelka 10/30/2015 16:22' prior: 0!
I represent data that are sent to a server.

When GTEventPacking object asks for data to send, the data are packed and added to #data:, then there are other general information about the bundle, e.g. computer id. I keep all those information. I can also unpack the data using #unpackedData.

For the simple backward compatibility, all my data are stored in a dictionary.

For the Collaborators Part:  
- GTEventCollector creates instance of me and adds additional information about sent data, see GTEventCollector>>#updateAnnouncement:
- GTEventPacking serialize #data and me in order to send me to a server

Public API and Key Messages

- unpackedData   
 
Internal Representation and Key Implementation Points.

    Instance Variables
	dictionary:		<Dictionary>
!

!GTEventAnnouncement class methodsFor: 'examples' stamp: 'JurajKubelka 10/11/2015 14:26'!
gtExampleAnnouncement
	<gtExample>
	^ self new
			data: #[ 1 2 3 4 ];
			timestamp: DateAndTime now;
			imageHash: SmalltalkImage current imagePath hash;
			sessionCreationTime: SmalltalkImage current session creationTime;
			imageVersion: SystemVersion current version;
			latestUpdate: SystemVersion current highestUpdate;
			yourself! !

!GTEventAnnouncement class methodsFor: 'examples' stamp: 'JurajKubelka 10/9/2015 15:26'!
gtExampleEmptyAnnouncement
	<gtExample>
	^ self new! !

!GTEventAnnouncement methodsFor: 'updating' stamp: 'JurajKubelka 10/9/2015 15:26'!
addInfoFor: aGTSpotterRecorderEvent 
	dictionary keysAndValuesDo: [ :key :value |
		key = #data ifFalse: [ 
			aGTSpotterRecorderEvent infoAt: key put: value ] ]! !

!GTEventAnnouncement methodsFor: 'converting' stamp: 'JurajKubelka 10/9/2015 15:26'!
associations
	^ dictionary 
			ifNil: [ #() ]
			ifNotNil: [ dictionary associations ]! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
at: key
	^ self dictionary at: key ifAbsent: [ nil ]! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
at: key put: value
	^ self dictionary at: key put: value! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
computerUUID
	^ self at: #computerUUID! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
computerUUID: anObject
	self at: #computerUUID put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
data
	^ self at: #data! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
data: aByteArray 
	self at: #data put: aByteArray! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
dictionary
	^ dictionary ifNil: [ dictionary := Dictionary new ]! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
eventRecorderVersion
	^ self at: #eventRecorderVersion! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
eventRecorderVersion: anObject
	self at: #eventRecorderVersion put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/31/2015 18:51'!
events
	| events |
	events := self unpackedData.
	events do: [ :each | self addInfoFor: each ].
	^ events! !

!GTEventAnnouncement methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 10/9/2015 15:26'!
gtInspectorItems
	^ OrderedCollection withAll: self associations! !

!GTEventAnnouncement methodsFor: 'gt-inspector-extension' stamp: 'JurajKubelka 10/9/2015 15:26'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: -1>  
	^ (composite table)
		title: 'Info';
		display: [ :anEvent | anEvent gtInspectorItems ];
		children: [ :each | 
			(each value isDictionary) 
				ifTrue: [ each value associations ] 
				ifFalse: [ (each value isArray and: [ each value allSatisfy: #isDictionary ])
						ifTrue: [ each value collectWithIndex: [:x :i | i -> x ] ]
						ifFalse: [ #() ] ] ];
		column: 'Key' 
			evaluated: [:each | GTObjectPrinter new asTruncatedTextFrom: each key ];
		column: 'Value' 
			evaluated: [:each | GTObjectPrinter new asTruncatedTextFrom: each value ];
		showOnly: 50! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
imageHash
	^ self at: #imageHash! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
imageHash: anObject
	self at: #imageHash put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
imageVersion
	^ self at: #imageVersion! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
imageVersion: anObject
	self at: #imageVersion put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
latestUpdate
	^ self at: #latestUpdate! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
latestUpdate: anObject
	self at: #latestUpdate put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 14:11'!
migrationRules
	^ self dictionary at: #migrationRules ifAbsent: [ nil ]! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 14:13'!
migrationRules: aCollectionOrNil
	^ self dictionary at: #migrationRules put: aCollectionOrNil! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 23:25'!
safeUnpackedData
	"On any error, returns empty collection"
	^ [ self unpackedData ] on: Error do: [ #() ]! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 15:31'!
serializedBy
	^ self dictionary at: #serializedBy ifAbsent: [ GTFuelPacking name ]! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 15:16'!
serializedBy: anObject
	self at: #serializedBy put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
sessionCreationTime
	^ self at: #sessionCreationTime! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
sessionCreationTime: anObject
	self at: #sessionCreationTime put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:28'!
sessionUUID
	^ self at: #sessionUUID! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:28'!
sessionUUID: anObject
	self at: #sessionUUID put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
timestamp
	^ self at: #timestamp! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:26'!
timestamp: anObject
	self at: #timestamp put: anObject! !

!GTEventAnnouncement methodsFor: 'accessing' stamp: 'JurajKubelka 10/31/2015 21:35'!
unpackedData
	^ self data ifNotNil: [ :aData | 	
			self serializedBy asClass new
				migrations: self migrationRules;
				unpack: aData ]! !

Announcement subclass: #PrivacyChangedAnnoucement
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Privacy'!

!PrivacyChangedAnnoucement commentStamp: 'JurajKubelka 12/18/2015 12:48' prior: 0!
I announce  changes on Privacy settings in SystemAnnouncer uniqueInstance.!

Object subclass: #GTEventBundle
	instanceVariableNames: 'url category data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventBundle commentStamp: 'AndreiChis 10/13/2015 00:34' prior: 0!
I associate a piece of data that should be delivered to a remote server with the url of that remote server.

Instances of me should be sent to a GTEventDelivery.

!

!GTEventBundle methodsFor: 'visitor' stamp: 'JurajKubelka 11/18/2015 14:09'!
accept: aVisitor 
	^ aVisitor visitSingleBundle: self! !

!GTEventBundle methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 09:30'!
category
	^ category! !

!GTEventBundle methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 09:30'!
category: anObject
	category := anObject! !

!GTEventBundle methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:56'!
data
	^ data! !

!GTEventBundle methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:56'!
data: anObject
	data := anObject! !

!GTEventBundle methodsFor: 'accessing - computed' stamp: 'JurajKubelka 11/15/2015 11:12'!
entity
	^ ZnByteArrayEntity bytes: self data! !

!GTEventBundle methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:56'!
url
	^ url! !

!GTEventBundle methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:56'!
url: anObject
	url := anObject! !

Object subclass: #GTEventCollector
	instanceVariableNames: 'url category recordedEvents registry creationTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventCollector commentStamp: 'JurajKubelka 11/17/2015 14:35' prior: 0!
I represent a collection of data that should be sent to a specific URL. 

I am responsible for collecting data (#add: method) and for giving collected data to GTEventPacking object (#bundle method).

You should consider to set #category. This value is used for separating data on the server to directories. That way you will then easier receive your collected data. If you creates a subclass of me, you can override #defaultCategory method.

You should also consider set #occupant:. This could be an object that uses me and feeds me with data. I keep this occupant weakly. So, if the object is removed from the memory, GTEventRecorder knows that I can be removed too. 

I collaborate with GTEventPacking object that receives collected data and URL, serilize the collected data, and creates GTEventBundle object. My subclasses can handle differently what data to send together to server by rewritting #bundle method.

Public API and Key Messages

- bundle   
- url
- category
- occupant:
- register

Example for playing with me:
	GTEventCollector new
		category: #testingForFun;
		occupant: nil "object that represents your application";
		add: 42;
		add: 'Hallo Pharo user';
		yourself.

Example for serious use:
	GTEventCollector new
		category: #testingForReal;
		occupant: nil "object that represents your application, if nil, the collector will be removed from GTEventRecorder object once collected data are sent";
		register; "please do not register just for fun, because we collects those data"
		yourself
 
Internal Representation and Key Implementation Points.

    Instance Variables
	recordedEvents:		<WaitfreeQueue>
	url:		<ZnUrl>
!

!GTEventCollector methodsFor: 'adding' stamp: 'JurajKubelka 10/9/2015 12:14'!
add: anEvent 
	^ recordedEvents nextPut: anEvent! !

!GTEventCollector methodsFor: 'adding' stamp: 'JurajKubelka 12/18/2015 12:43'!
addIfAvailable: anEvent
	Privacy sendDiagnosticsAndUsageData ifFalse: [ ^ anEvent ].	
	^ self add: anEvent! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 13:59'!
bundle
	^ self bundle: OrderedCollection new.! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 13:58'!
bundle: aCollection
	recordedEvents flush: [ :eachEvent | aCollection addLast: eachEvent ].
	^ aCollection! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 11/13/2015 22:29'!
category
	^ category ifNil: [ category := self defaultCategory ]! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 11/13/2015 17:11'!
category: aString 
	^ category := aString! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 11/17/2015 11:22'!
creationTime
	^ creationTime! !

!GTEventCollector methodsFor: 'configuration' stamp: 'JurajKubelka 11/13/2015 22:30'!
defaultCategory
	^ #unknown! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 11/16/2015 19:32'!
flush
	"Be careful when using it. You will lost all data."
	recordedEvents flush! !

!GTEventCollector methodsFor: 'initialization' stamp: 'JurajKubelka 11/17/2015 11:16'!
initialize
	super initialize.	
	recordedEvents := WaitfreeQueue new.
	creationTime := DateAndTime now.! !

!GTEventCollector methodsFor: 'testing' stamp: 'JurajKubelka 11/16/2015 19:34'!
isLiberated
	"When I answer true, I can be removed from a GTEventRecorder object."
	^ self registry isLiberated and: [ recordedEvents isEmpty ]! !

!GTEventCollector methodsFor: 'actions' stamp: 'JurajKubelka 11/16/2015 19:56'!
liberate
	"Liberate me from the occupant so I can be removed from GTEventRecorder object in the next delivery cycle."
	^ self registry liberate.! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 11/16/2015 19:37'!
occupant: anObject 
	"anObject sends me data. I weakly keeps anObject and once it is released, I can be removed from GTEventRecorder object."
	self registry object: anObject! !

!GTEventCollector methodsFor: 'printing' stamp: 'JurajKubelka 10/11/2015 19:50'!
printOn: aStream
	super printOn: aStream.
	aStream
		space;
		nextPut: $[;
		print: recordedEvents size;
		space;
		nextPutAll: ('item' asPluralBasedOn: recordedEvents size);
		nextPut: $]! !

!GTEventCollector methodsFor: 'initialization' stamp: 'JurajKubelka 11/16/2015 19:11'!
register
	"Register to the global GTEventRecorder."
	GTEventRecorder uniqueInstance addCollector: self.! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 11/16/2015 19:34'!
registry
	^ registry ifNil: [ registry := GTEventRegistry new ]! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 12:12'!
size
	^ recordedEvents size! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 14:20'!
url
	^ url ifNil: [ url := GTEventRecorderSettings url ]! !

!GTEventCollector methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 12:06'!
url: anObject
	url := anObject! !

Object subclass: #GTEventCollectors
	instanceVariableNames: 'mutex collectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventCollectors commentStamp: 'JurajKubelka 10/13/2015 22:53' prior: 0!
I represent a collection of GTEventCollector objects. GTEventRecorder uses me and I am responsible for adding and removing the collectors.

Public API and Key Messages

- add:  
- remove:
 
Internal Representation and Key Implementation Points.

    Instance Variables
	collectors:		<OrderedCollection>
	mutex:		<Mutex>
!

!GTEventCollectors methodsFor: 'adding' stamp: 'JurajKubelka 10/30/2015 15:02'!
add: aGTEventCollector
	^ mutex critical: [ collectors add: aGTEventCollector ]! !

!GTEventCollectors methodsFor: 'enumerating' stamp: 'JurajKubelka 11/17/2015 11:27'!
do: oneArgBlock
	mutex critical: [ (collectors sorted: [ :a :b | a creationTime < b creationTime ]) do: oneArgBlock ]! !

!GTEventCollectors methodsFor: 'gt-inspector-extension' stamp: 'AndreiChis 11/16/2015 11:06'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: 1>
	^ collectors ifNotNil: [ 
			(collectors gtInspectorItemsIn: composite)
				title: 'Collectors';
				display: [ collectors asOrderedCollection ] ]! !

!GTEventCollectors methodsFor: 'initialization' stamp: 'JurajKubelka 10/30/2015 15:02'!
initialize
	super initialize.
	mutex := Mutex new.
	collectors := IdentitySet new.! !

!GTEventCollectors methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 15:49'!
numberOfEvents
	^ mutex critical: [ 
		collectors 
			ifEmpty: 0
			ifNotEmpty: [ collectors sum: #size ] ]! !

!GTEventCollectors methodsFor: 'printing' stamp: 'JurajKubelka 10/11/2015 19:52'!
printOn: aStream
	super printOn: aStream.
	aStream
		space;
		nextPut: $[;
		print: self size;
		space;
		nextPutAll: ('collector' asPluralBasedOn: self size);
		nextPutAll: ', ';
		print: self numberOfEvents;
		space;
		nextPutAll: ('event' asPluralBasedOn: self numberOfEvents);
		nextPut: $]! !

!GTEventCollectors methodsFor: 'removing' stamp: 'JurajKubelka 10/11/2015 15:57'!
remove: aGTEventCollector 
	^ mutex critical: [ collectors remove: aGTEventCollector ifAbsent: nil ]! !

!GTEventCollectors methodsFor: 'removing' stamp: 'JurajKubelka 11/16/2015 19:51'!
removeLiberatedCollectors
	mutex critical: [ | liberated |
		liberated := collectors select: #isLiberated.
		collectors removeAll: liberated ]! !

!GTEventCollectors methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 10:33'!
size
	^ mutex critical: [ collectors size ]! !

Object subclass: #GTEventConsolidate
	instanceVariableNames: 'collectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventConsolidate commentStamp: 'JurajKubelka 11/17/2015 14:03' prior: 0!
I consolidate GTEventCollector objects into one or more collectors according to their URL and category. That way we can pack together data, that belongs to the same application (= URL and category) and we can save them together on the server side as one bundle.

For the Collaborators Part: 
- GTEventCollector objects that I consolidate
- GTEventRecorder that uses me before packing process.

Public API and Key Messages

- collectors: set a collection of collectors that should be consolidated
- consolidated returns collection of GTEventCollector objects.
 
Internal Representation and Key Implementation Points.

    Instance Variables
	collectors:		<OrderedCollection>
!

!GTEventConsolidate methodsFor: 'accessing' stamp: 'JurajKubelka 11/17/2015 13:33'!
collectors: aCollection 
	collectors := aCollection! !

!GTEventConsolidate methodsFor: 'accessing - computed' stamp: 'JurajKubelka 11/17/2015 13:40'!
consolidated
	| consolidatedCollectors |
	consolidatedCollectors := OrderedDictionary new.
	collectors do: [ :eachCollector | | aCollector |
		aCollector := consolidatedCollectors 
								at: (eachCollector url printString, '/', eachCollector category) asSymbol
								ifAbsentPut: [ GTEventCollector new 
														url: eachCollector url; 
														category: eachCollector category;
														yourself ].
		eachCollector bundle do: [ :eachEvent | aCollector add: eachEvent ] ].
	^ consolidatedCollectors asArray! !

Object subclass: #GTEventDelivery
	instanceVariableNames: 'mutex process delay permission recorder queue preSuspendingAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventDelivery commentStamp: 'AndreiChis 10/13/2015 00:35' prior: 0!
I know how to deliver usage and diagnostic data to a remote server.

I start and maintain a process for sending data on a regular inteval (#standardDuration).
I maintain a queue with the data that needs to be send. If a delivert fails I re-add the data to the queue.

Data that is added to my queue must be packaged in an object of type GTEventBundle.  I use the url specified by the bundle.
I send the recorder the message #pack to notify it that I enter the delivery cycle.

Public API and Key Messages

- #activate initializes the delivery process; clients must call this method; it's not called automatically on object creation. 
- #deliveryCycle implements  the main logic of the delivery process!

!GTEventDelivery methodsFor: 'activation' stamp: 'JurajKubelka 10/9/2015 21:04'!
activate
	"Initialize a delivery process if permitted and necessary."
	^ mutex critical: [ 
			self hasPermission ifFalse: [ ^ self ].
			self isInactive ifTrue: [ self initializeProcess ].
			process ]! !

!GTEventDelivery methodsFor: 'adding' stamp: 'AndreiChis 10/13/2015 00:21'!
add: aGTEventBundle
	queue nextPut: aGTEventBundle! !

!GTEventDelivery methodsFor: 'announce' stamp: 'JurajKubelka 11/13/2015 17:36'!
announceSuspendingDelivery
	preSuspendingAction ifNotNil: #value! !

!GTEventDelivery methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 20:18'!
announcer
	^ self recorder ifNotNil: #announcer! !

!GTEventDelivery methodsFor: 'delivery' stamp: 'JurajKubelka 11/20/2015 08:46'!
bundles
	"Be careful about using this method. You will loose data if you call it externaly."
	| eventBundles |
	eventBundles := OrderedCollection new.
	queue flush: [ :eachGTEventBundle | eventBundles addLast: eachGTEventBundle ].
	^ eventBundles! !

!GTEventDelivery methodsFor: 'deactivation' stamp: 'JurajKubelka 10/9/2015 11:27'!
deactivate
	^ mutex critical: [ | oldProcess |
		self withdrawPermission.
		delay ifNotNil: [ :aDelay | aDelay delaySemaphore signal ].
		oldProcess := process.
		process := nil.
		oldProcess ]! !

!GTEventDelivery methodsFor: 'delivery' stamp: 'JurajKubelka 11/15/2015 09:45'!
deliver: aBoolean
	| bundles |
	bundles := GTEventMultiBundleFactory fromBundles: self bundles.
	bundles do: [ :eachGTEventMultiBundle |
		self deliver: eachGTEventMultiBundle signalProgress: aBoolean ]! !

!GTEventDelivery methodsFor: 'delivery' stamp: 'JurajKubelka 11/18/2015 10:18'!
deliver: aGTEventMultiBundle signalProgress: aBoolean
	^ (ZnClient new
			beOneShot;
			timeout: 5;
			signalProgress: aBoolean;
			numberOfRetries: 0;
			url: aGTEventMultiBundle url;
			enforceHttpSuccess: true;
			ifFail: [ :exception | queue nextPut: aGTEventMultiBundle ];
			entity: aGTEventMultiBundle entity;
			put;
			response)! !

!GTEventDelivery methodsFor: 'initialization' stamp: 'JurajKubelka 10/9/2015 21:17'!
deliveryCycle
	delay := self standardDuration asDelay.
	self announceSuspendingDelivery.
	delay wait.
	delay := nil.
	self hasPermission ifTrue: [ self packAndDeliver: false ]! !

!GTEventDelivery methodsFor: 'testing' stamp: 'JurajKubelka 10/9/2015 11:16'!
hasPermission
	^ permission! !

!GTEventDelivery methodsFor: 'initialization' stamp: 'JurajKubelka 11/13/2015 17:36'!
initialize
	super initialize.
	mutex := Mutex new.
	queue := WaitfreeQueue new.
	permission := true.
	preSuspendingAction := nil.! !

!GTEventDelivery methodsFor: 'initialization' stamp: 'JurajKubelka 10/11/2015 16:13'!
initializeProcess
	process := [ [ self hasPermission ] whileTrue: [ self deliveryCycle ] ]
		forkAt: Processor userBackgroundPriority
		named: self printProcessName! !

!GTEventDelivery methodsFor: 'testing' stamp: 'JurajKubelka 10/9/2015 10:53'!
isActive
	^ self isInactive not! !

!GTEventDelivery methodsFor: 'testing' stamp: 'JurajKubelka 10/11/2015 16:15'!
isGlobalRecorder
	^ self recorder 
			ifNil: false
			ifNotNil: #isGlobalRecorder! !

!GTEventDelivery methodsFor: 'testing' stamp: 'JurajKubelka 10/9/2015 20:56'!
isInactive
	^ mutex critical: [ 
		process isNil or: [
			process isTerminated or: [
				process isTerminating ] ] ]! !

!GTEventDelivery methodsFor: 'delivery' stamp: 'JurajKubelka 10/9/2015 11:30'!
now
	"for testing purpose"
	delay ifNotNil: [ :aDelay | aDelay delaySemaphore signal. aDelay unschedule. ]! !

!GTEventDelivery methodsFor: 'activation' stamp: 'JurajKubelka 10/9/2015 11:23'!
offerPermission
	mutex critical: [ permission := true ]! !

!GTEventDelivery methodsFor: 'delivery' stamp: 'JurajKubelka 10/9/2015 12:24'!
packAndDeliver: aBoolean.
	recorder pack.
	self deliver: aBoolean.
	! !

!GTEventDelivery methodsFor: 'initialization' stamp: 'JurajKubelka 11/13/2015 22:22'!
preSuspendingAction: aBlockClosure 
	preSuspendingAction := aBlockClosure! !

!GTEventDelivery methodsFor: 'printing' stamp: 'JurajKubelka 10/11/2015 20:24'!
printOn: aStream
	super printOn: aStream.
	aStream
		space;
		nextPut: $[;
		print: queue size;
		space;
		nextPutAll: ('item' asPluralBasedOn: queue size).
	self isActive
		ifTrue: [ 
			delay 
				ifNil: [ aStream nextPutAll: 'process active, but delay is nil!!' ]
				ifNotNil: [ :aDelay |
					aStream
						nextPutAll: ', next cycle in ';
						print: (aDelay millisecondsToGo milliSeconds roundTo: 1 second);
						nextPutAll: ' sec' ] ]
		ifFalse: [ 
			aStream nextPutAll: ', inactive' ].
	aStream		
		nextPut: $]! !

!GTEventDelivery methodsFor: 'printing' stamp: 'JurajKubelka 10/11/2015 16:13'!
printProcessName
	^ String streamContents: [ :aStream |
		self printProcessNameOn: aStream ]! !

!GTEventDelivery methodsFor: 'printing' stamp: 'JurajKubelka 10/11/2015 16:13'!
printProcessNameOn: aStream
	aStream
		nextPutAll: self className.
	self isGlobalRecorder
		ifTrue: [ aStream nextPutAll: ' global ' ]
		ifFalse: [ aStream nextPutAll: ' a ' ].
	aStream nextPutAll: 'delivery process'! !

!GTEventDelivery methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 12:20'!
recorder
	^ recorder! !

!GTEventDelivery methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 12:20'!
recorder: anObject
	recorder := anObject! !

!GTEventDelivery methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:46'!
size
	^ queue size! !

!GTEventDelivery methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 11:11'!
standardDuration
	^ 20 minutes! !

!GTEventDelivery methodsFor: 'deactivation' stamp: 'JurajKubelka 10/9/2015 11:23'!
withdrawPermission
	mutex critical: [ permission := false ]! !

Object subclass: #GTEventMultiBundle
	instanceVariableNames: 'url bundles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventMultiBundle commentStamp: 'JurajKubelka 11/15/2015 18:03' prior: 0!
I keep all GTEventBundle objects that should be delivered to the same remote server. That way, all data can be sent to the server at one HTTP request.

By calling #entity, you receive ZnMultiPartFormDataEntity object that contains ZnMimePart objects in a sequence 'category', 'data', 'category', 'data', etc. Server is then responsible for spliting this object into the parts and store each data in to a right position (directory).

For the Collaborators Part: 
- GTEventBundle: I keep collection of those objects and asks them for a ZnEntity object calling #entity.
- GTEventMultiBundleFactory knows how to create instances.
- GTEventDelivery knows how to send my #entity to a server.

Internal Representation and Key Implementation Points.

    Instance Variables
	bundles:		<Collection>
	url:			<ZnUrl>
!

!GTEventMultiBundle methodsFor: 'visitor' stamp: 'JurajKubelka 11/18/2015 14:10'!
accept: aVisitor 
	^ aVisitor visitMultiBundle: self! !

!GTEventMultiBundle methodsFor: 'adding' stamp: 'JurajKubelka 11/15/2015 10:14'!
add: aGTEventBundle 
	^ self bundles addLast: aGTEventBundle! !

!GTEventMultiBundle methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 10:14'!
bundles
	^ bundles ifNil: [ bundles := OrderedCollection new ]! !

!GTEventMultiBundle methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 11:02'!
entity
	| entity |
	entity := ZnMultiPartFormDataEntity new.
	self updateEntity: entity.
	^ entity! !

!GTEventMultiBundle methodsFor: 'private' stamp: 'JurajKubelka 11/18/2015 10:19'!
updateEntity: aZnMultiPartFormDataEntity 
	bundles ifNil: [ ^ self ].
	self bundles do: [ :eachEventBundle |
		aZnMultiPartFormDataEntity 
			addPart: (ZnMimePart fieldName: 'category' value: eachEventBundle category);
			addPart: (ZnMimePart fieldName: 'data' entity: eachEventBundle entity) ]! !

!GTEventMultiBundle methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 09:43'!
url
	^ url! !

!GTEventMultiBundle methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 09:43'!
url: anObject
	url := anObject! !

Object subclass: #GTEventMultiBundleFactory
	instanceVariableNames: 'multiBundles'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventMultiBundleFactory commentStamp: 'JurajKubelka 11/15/2015 18:03' prior: 0!
I know how to create GTEventMultiBundle object from a collection of GTEventBundle objects.!

!GTEventMultiBundleFactory class methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 10:57'!
default
	^ self new! !

!GTEventMultiBundleFactory class methodsFor: 'public' stamp: 'JurajKubelka 11/15/2015 10:57'!
fromBundles: aCollection 
	^ self default fromBundles: aCollection! !

!GTEventMultiBundleFactory methodsFor: 'public' stamp: 'JurajKubelka 11/18/2015 14:04'!
fromBundles: aCollection 
	multiBundles := OrderedDictionary new.
	aCollection do: [ :aSingleOrMultiBundle | self synthetize: aSingleOrMultiBundle.].
	^ self multiBundles! !

!GTEventMultiBundleFactory methodsFor: 'accessing' stamp: 'JurajKubelka 11/18/2015 14:05'!
multiBundles
	^ multiBundles asOrderedCollection! !

!GTEventMultiBundleFactory methodsFor: 'private' stamp: 'JurajKubelka 11/18/2015 14:06'!
synthetize: aSingleOrMultiBundle
	aSingleOrMultiBundle accept: self! !

!GTEventMultiBundleFactory methodsFor: 'visitor' stamp: 'JurajKubelka 11/18/2015 14:08'!
visitMultiBundle: aMultiBundle
	aMultiBundle bundles do: [ :eachSingleBundle |
		self visitSingleBundle: eachSingleBundle ]! !

!GTEventMultiBundleFactory methodsFor: 'visitor' stamp: 'JurajKubelka 11/18/2015 14:07'!
visitSingleBundle: aSingleBundle
	| multiBundle |
	multiBundle := multiBundles 
							at: aSingleBundle url 
							ifAbsentPut: [ GTEventMultiBundle new url: aSingleBundle url; yourself ].
	multiBundle add: aSingleBundle! !

Object subclass: #GTEventPacking
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventPacking commentStamp: 'JurajKubelka 10/13/2015 23:00' prior: 0!
I represent an abstract packing system that is able to receive objects and pack them into a binary data (ByteArray).

Others can ask me for serializing an object by sending me #pack: or for materializing a ByteArray by sending me #unpack:. On the class side, you can find out #fuel and #ston methods, two available packing strategies. The method #version keeps current packing version and changelog. 

The main users are GTEventRecorder (for packing) and GTEventAnnouncement (for unpacking).

Public API and Key Messages

- pack:
- unpack:
- version
!

!GTEventPacking class methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 18:36'!
all
	^ {self ston. self fuel}! !

!GTEventPacking class methodsFor: 'instance creation' stamp: 'JurajKubelka 10/30/2015 09:23'!
default
	^ self ston! !

!GTEventPacking class methodsFor: 'instance creation' stamp: 'JurajKubelka 10/10/2015 15:19'!
fuel
	^ GTFuelPacking new! !

!GTEventPacking class methodsFor: 'instance creation' stamp: 'JurajKubelka 10/10/2015 15:19'!
ston
	^ GTStonPacking new! !

!GTEventPacking methodsFor: 'private' stamp: 'JurajKubelka 10/10/2015 15:17'!
materialize: aByteArray
	^ self subclassResponsibility! !

!GTEventPacking methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 14:16'!
migrations: anObject 
	"Ignore by default. Subclasses can overwrite it."! !

!GTEventPacking methodsFor: 'instance creation' stamp: 'JurajKubelka 11/17/2015 13:51'!
newAnnouncement
	^ GTEventAnnouncement new! !

!GTEventPacking methodsFor: 'private' stamp: 'JurajKubelka 11/17/2015 13:50'!
newAnnouncementFor: aGTEventCollector
	^ self 
		updateAnnouncement: self newAnnouncement 
		withData: (self serialize: aGTEventCollector bundle).! !

!GTEventPacking methodsFor: 'packing' stamp: 'JurajKubelka 11/15/2015 09:30'!
pack: aGTEventCollector
	^ GTEventBundle new
			url: aGTEventCollector url;
			category: aGTEventCollector category;
			data: (self packCollector: aGTEventCollector);
			yourself! !

!GTEventPacking methodsFor: 'private' stamp: 'JurajKubelka 10/14/2015 09:15'!
packCollector: aGTEventCollector
	^ self serialize: (self newAnnouncementFor: aGTEventCollector)! !

!GTEventPacking methodsFor: 'private' stamp: 'JurajKubelka 10/10/2015 15:18'!
serialize: anObject
	^ self subclassResponsibility! !

!GTEventPacking methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 18:37'!
technologyName
	"It returns a string that identify packing technology."
	^ self subclassResponsibility! !

!GTEventPacking methodsFor: 'packing' stamp: 'JurajKubelka 10/9/2015 15:38'!
unpack: aByteArray
	^ self materialize: aByteArray! !

!GTEventPacking methodsFor: 'private' stamp: 'JurajKubelka 10/14/2015 09:11'!
updateAnnouncement: aGTEventAnnouncement withData: aByteArray
	^ aGTEventAnnouncement
			data: aByteArray;
			serializedBy: self class name;
			eventRecorderVersion: self version;
			timestamp: DateAndTime now;
			imageHash: SmalltalkImage current imagePath hash;
			imageVersion: SystemVersion current version;
			latestUpdate: SystemVersion current highestUpdate;
			computerUUID: GlobalIdentifier uniqueInstance computerUUID asString;
			sessionUUID: SmalltalkImage current session sid;
			sessionCreationTime: SmalltalkImage current session creationTime;
			yourself! !

!GTEventPacking methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 18:52'!
version
	^ 4
	
"
Changes to version 4:
	- It introduces GTEventMultiBundle object that is used for sending several GTEVentBundle objects to the same server in one HTTP request
	- GTStonPacking returns ByteArray instead of ByteString
Changes to version 3:
	- GTSpotterEventRecorder rewritten to GTEventRecorder.
	- It uses STON instead of FUEL serializing engine.
		- FUEL engine is still available using GTFuelPacking object
Changes to version 2 (FUEL):
	- GTSpotterEventRecorder>>timestamp keeps DateAndTime object. 
	  Before, it was an integer as unix time.
"! !

GTEventPacking subclass: #GTFuelPacking
	instanceVariableNames: 'migrations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTFuelPacking commentStamp: 'JurajKubelka 10/13/2015 23:01' prior: 0!
I serialize or materialize objects using FUEL framework. Create instance of me calling GTEventPacking fuel.!

!GTFuelPacking methodsFor: 'adding' stamp: 'JurajKubelka 10/30/2015 13:56'!
addMigration: oneArgBlock 
	^ self migrations addLast: oneArgBlock! !

!GTFuelPacking methodsFor: 'initialization' stamp: 'JurajKubelka 10/30/2015 14:39'!
initializeMaterializer: aFLMaterializer 
	self migrations do: [ :eachMigration | eachMigration value: aFLMaterializer ]! !

!GTFuelPacking methodsFor: 'initialization' stamp: 'JurajKubelka 10/30/2015 14:36'!
initializeMigrations
	| pragmas |
	pragmas := Pragma allNamed: #fuelMaterializerMigration in: self class.
	migrations := pragmas 
						collect: [ :eachPragma | self perform: eachPragma selector ] 
						as: OrderedCollection.! !

!GTFuelPacking methodsFor: 'private' stamp: 'JurajKubelka 11/1/2015 10:44'!
materialize: aByteArray
	| materializer stream gzstream materialization |
	stream := aByteArray asByteArray readStream. 
	gzstream := GZipReadStream on: stream.
	materializer := FLMaterializer newDefault.
	self initializeMaterializer: materializer.
	materialization := materializer materializeFrom: gzstream.
	gzstream close.
	stream close.
	^ materialization root.! !

!GTFuelPacking methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 14:19'!
migrations
	migrations ifNil: [ self initializeMigrations ].
	^ migrations! !

!GTFuelPacking methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 14:14'!
migrations: aCollectionOrNil
	migrations := aCollectionOrNil! !

!GTFuelPacking methodsFor: 'private' stamp: 'JurajKubelka 10/10/2015 15:08'!
serialize: anObject
	| serializer stream gzstream |
	stream := ByteArray new writeStream binary. 
	gzstream := GZipWriteStream on: stream.
	serializer := FLSerializer newDefault.
	serializer at: #recorderVersion putAdditionalObject: self version.
	serializer stream: gzstream.
	serializer serialize: anObject.
	gzstream close.
	stream close.
	^ stream contents.! !

!GTFuelPacking methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 18:37'!
technologyName
	^ 'fuel'! !

GTEventPacking subclass: #GTStonPacking
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTStonPacking commentStamp: 'JurajKubelka 10/13/2015 23:01' prior: 0!
I serialize or materialize objects using STON framework. Create instance of me calling GTEventPacking ston.!

!GTStonPacking methodsFor: 'private' stamp: 'JurajKubelka 11/1/2015 10:12'!
materialize: aByteArray
	| stream gzstream |
	stream := aByteArray asString readStream.
	gzstream := GZipReadStream on: stream.
	^ [ STON fromStream: gzstream ] ensure: [ gzstream close. stream close ]! !

!GTStonPacking methodsFor: 'private' stamp: 'JurajKubelka 11/15/2015 18:32'!
serialize: anObject
	^ (String streamContents: [ :stream | | gzstream |
		gzstream := GZipWriteStream on: stream.
		[ STON put: anObject onStreamPretty: gzstream ]
			ensure: [ gzstream close ] ]) asByteArray! !

!GTStonPacking methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 18:37'!
technologyName
	^ 'ston'! !

Object subclass: #GTEventRecorder
	instanceVariableNames: 'collectors delivery packing privacy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

GTEventRecorder class
	instanceVariableNames: 'uniqueInstance'!

!GTEventRecorder commentStamp: 'JurajKubelka 10/13/2015 22:36' prior: 0!
I represent an event recorder, an object that collects data and regularly sends them to a server. The data could represent user activity, tool usage, or any other statistic usefule for further analysis. 

I cover data collectors, packing, and delivery objects the way they cooperate together and the collected data are regularly send to server(s). It is not mandatory, but it is supposed that I represent a Singleton pattern.

I collaborate with GTEventCollector objects, that are responsible for collecting data. I keep any number of those objects and whenever GTEventDelivery asks me for collected data, GTEventPacking object packs data from each GTEventCollector and hand them as collection of GTEventBundle objects to the GTEventDelivery. GTEventDelivery sends the bundles to server(s).

Public API and Key Messages

- addCollector: it adds GTEventCollector object and activates delivery process if inactive   
- removeCollector: it removes  GTEventCollector object and deactivates delivery process if there are no other collectors.
- activateDelivery: activates delivery process
- deactivateDelivery: deactivates delivery process

Internal Representation and Key Implementation Points.

    Instance Variables
	announcer:		<Annoucner>
	collectors:		<GTEventCollectors>
	delivery:		<GTEventDelivery>
	packing:		<GTEventPacking>
	privacy:		<true|false|ClosureBlock>
!

GTEventRecorder class
	instanceVariableNames: 'uniqueInstance'!

!GTEventRecorder class methodsFor: 'cleanup' stamp: 'JurajKubelka 10/11/2015 14:45'!
cleanUp
	self reset.! !

!GTEventRecorder class methodsFor: 'initialization' stamp: 'JurajKubelka 12/18/2015 12:15'!
initializeUniqueInstance
	uniqueInstance ifNotNil: [ ^ uniqueInstance ].
	uniqueInstance := self new.
	self registerInterestToSystemAnnouncement.
	^ uniqueInstance! !

!GTEventRecorder class methodsFor: 'testing' stamp: 'JurajKubelka 10/11/2015 16:16'!
isGlobalRecorder: aGTEventRecorder
	^ uniqueInstance 
			ifNil: false
			ifNotNil: [ :anInstance | anInstance == aGTEventRecorder  ]! !

!GTEventRecorder class methodsFor: 'initialization' stamp: 'JurajKubelka 12/18/2015 12:22'!
registerInterestToSystemAnnouncement
	<systemEventRegistration>
	uniqueInstance ifNil: [ ^ self ].
	SystemAnnouncer uniqueInstance 
		unsubscribe: uniqueInstance;
		when: PrivacyChangedAnnoucement
			send: #privacyChanged 
			to: uniqueInstance.! !

!GTEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 12/18/2015 12:19'!
reset
	"Stop and remove the process for sending usage data. Delete all recorder that from the queue."
	uniqueInstance ifNil: [ ^ self ].
	SystemAnnouncer uniqueInstance unsubscribe: uniqueInstance.
	uniqueInstance deactivateDelivery ifNotNil: [ :aProcess | 
			[ aProcess terminate ] on: Error do: [ "ignore error" ] ].
	uniqueInstance := nil.! !

!GTEventRecorder class methodsFor: 'accessing' stamp: 'JurajKubelka 12/18/2015 12:20'!
uniqueInstance
	uniqueInstance ifNil: [ self initializeUniqueInstance ].
	^ uniqueInstance! !

!GTEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 10/9/2015 11:28'!
activateDelivery
	^ delivery activate! !

!GTEventRecorder methodsFor: 'adding' stamp: 'JurajKubelka 10/9/2015 10:56'!
addCollector: aGTEventCollector
	self activateDelivery.
	^ collectors add: aGTEventCollector! !

!GTEventRecorder methodsFor: 'testing' stamp: 'JurajKubelka 10/9/2015 14:49'!
canSendDiagnosticsAndUsageData
	^ privacy value! !

!GTEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 10/9/2015 11:28'!
deactivateDelivery
	^ delivery deactivate! !

!GTEventRecorder methodsFor: 'delivery' stamp: 'JurajKubelka 10/9/2015 11:29'!
deliverNow
	delivery now! !

!GTEventRecorder methodsFor: 'accessing' stamp: 'JurajKubelka 10/9/2015 15:47'!
delivery
	"Testing purpose."
	^ delivery! !

!GTEventRecorder methodsFor: 'configuration' stamp: 'JurajKubelka 11/13/2015 22:22'!
deliveryPreSuspendingAction: aBlockClosure 
	self delivery preSuspendingAction: aBlockClosure! !

!GTEventRecorder methodsFor: 'packing' stamp: 'JurajKubelka 11/17/2015 13:14'!
flush
	"Be careful!! You will lost all data."
	collectors do: #flush! !

!GTEventRecorder methodsFor: 'gt-inspector-extension' stamp: 'AndreiChis 11/16/2015 10:52'!
gtInspectorItemsIn: composite
	<gtInspectorPresentationOrder: 1>
	^ collectors ifNotNil: [ 
			(collectors gtInspectorItemsIn: composite) ]! !

!GTEventRecorder methodsFor: 'initialization' stamp: 'JurajKubelka 12/18/2015 12:39'!
initialize
	super initialize.
	privacy := [ Privacy sendDiagnosticsAndUsageData ].
	collectors := GTEventCollectors new.
	delivery := GTEventDelivery new.
	delivery recorder: self.
	packing := GTEventPacking default.
	self canSendDiagnosticsAndUsageData 
			ifTrue: [ delivery offerPermission ]
			ifFalse: [ delivery withdrawPermission ]! !

!GTEventRecorder methodsFor: 'testing' stamp: 'JurajKubelka 10/9/2015 10:44'!
isDeliveryActive
	^ delivery isActive! !

!GTEventRecorder methodsFor: 'testing' stamp: 'JurajKubelka 10/11/2015 16:15'!
isGlobalRecorder
	^ self class isGlobalRecorder: self! !

!GTEventRecorder methodsFor: 'convenient' stamp: 'JurajKubelka 10/9/2015 10:29'!
numberOfCollectors
	^ collectors size! !

!GTEventRecorder methodsFor: 'convenient' stamp: 'JurajKubelka 11/20/2015 08:23'!
numberOfDeliveryData
	^ delivery size! !

!GTEventRecorder methodsFor: 'configuration' stamp: 'JurajKubelka 10/9/2015 11:32'!
offerDeliveryPermission
	delivery offerPermission! !

!GTEventRecorder methodsFor: 'configuration' stamp: 'JurajKubelka 10/9/2015 14:51'!
offerPrivacy
	"testing purpose"
	privacy := true! !

!GTEventRecorder methodsFor: 'packing' stamp: 'JurajKubelka 11/17/2015 13:49'!
pack
	| allCollectors consolidatedCollectors |
	self canSendDiagnosticsAndUsageData ifFalse: [ ^ self flush; yourself ].
	allCollectors := OrderedCollection new: collectors size.
	collectors do: [ :eachCollector | allCollectors addLast: eachCollector ].
	consolidatedCollectors := GTEventConsolidate new collectors: allCollectors; consolidated.
	consolidatedCollectors do: [ :eachCollector | | bundle |
		bundle := packing pack: eachCollector.
		bundle data ifNotEmpty: [ delivery add: bundle ] ].
	self removeLiberatedCollectors.! !

!GTEventRecorder methodsFor: 'event handling' stamp: 'JurajKubelka 12/18/2015 12:50'!
privacyChanged
	self canSendDiagnosticsAndUsageData 
		ifTrue: [ self numberOfCollectors > 0 
						ifTrue: [ self activateDelivery ] ]
		ifFalse: [ self deactivateDelivery ]! !

!GTEventRecorder methodsFor: 'removing' stamp: 'JurajKubelka 10/11/2015 15:52'!
removeCollector: aGTEventCollector 
	| removedCollector |
	removedCollector := collectors remove: aGTEventCollector.
	collectors size = 0 ifTrue: [ self deactivateDelivery ].
	^ removedCollector! !

!GTEventRecorder methodsFor: 'removing' stamp: 'JurajKubelka 11/16/2015 19:48'!
removeLiberatedCollectors
	collectors removeLiberatedCollectors! !

!GTEventRecorder methodsFor: 'configuration' stamp: 'JurajKubelka 10/9/2015 11:24'!
withdrawDeliveryPermission
	delivery withdrawPermission! !

!GTEventRecorder methodsFor: 'configuration' stamp: 'JurajKubelka 10/9/2015 14:51'!
withdrawPrivacy
	"testing purpose"
	privacy := false! !

Object subclass: #GTEventRecorderSettings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Settings'!

GTEventRecorderSettings class
	instanceVariableNames: 'url'!

!GTEventRecorderSettings commentStamp: 'JurajKubelka 10/30/2015 16:57' prior: 0!
I represents Privacy settings. You can open Settings Browser and see Privacy group.

Execute: SettingBrowser open!

GTEventRecorderSettings class
	instanceVariableNames: 'url'!

!GTEventRecorderSettings class methodsFor: 'cleanup' stamp: 'JurajKubelka 10/11/2015 14:17'!
cleanUp
	"Clean settings."
	url := nil! !

!GTEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 14:13'!
stringUrl
	^ self url asString! !

!GTEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 14:13'!
stringUrl: aUrlString
	^ self url: aUrlString! !

!GTEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 14:12'!
url
	^ url ifNil: [ url := 'http://gc.dcc.uchile.cl.:8080/gt/events' asZnUrl ]! !

!GTEventRecorderSettings class methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 14:12'!
url: aZnUrl
	url := aZnUrl ifNotNil: [ aZnUrl asZnUrl ].! !

!GTEventRecorderSettings class methodsFor: 'settings' stamp: 'JurajKubelka 10/11/2015 14:14'!
urlSettingsOn: aBuilder 
	<systemsettings>	
	(aBuilder setting: #stringUrl)
		parent: #gt;
		target: self;
		description: 'The default URL for the Event Recorder server.' translated;
		label: 'Event Recorder URL address'
! !

Object subclass: #GTEventRegistry
	instanceVariableNames: 'weakHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventRegistry commentStamp: 'JurajKubelka 11/16/2015 19:59' prior: 0!
I am weak holder of an object that uses GTEventCollector. If the object disappears from object memory (or #liberate message is sent), GTEventCollector can be removed from a GTEventRecorder object.

For the Collaborators Part: GTEventCollector

Public API and Key Messages

- object: 
- liberate
- isLiberated

Internal Representation and Key Implementation Points.

    Instance Variables
	weakHolder:		<Object>
!

!GTEventRegistry methodsFor: 'initialization' stamp: 'JurajKubelka 11/16/2015 19:24'!
initialize
	super initialize.
	weakHolder := WeakArray new: 1.! !

!GTEventRegistry methodsFor: 'testing' stamp: 'JurajKubelka 11/16/2015 19:25'!
isLiberated
	^ (weakHolder at: 1) isNil! !

!GTEventRegistry methodsFor: 'actions' stamp: 'JurajKubelka 11/16/2015 19:54'!
liberate
	self object: nil.! !

!GTEventRegistry methodsFor: 'accessing' stamp: 'JurajKubelka 11/16/2015 19:24'!
object: anObject 
	weakHolder at: 1 put: anObject.! !

!GTEventRegistry methodsFor: 'printing' stamp: 'JurajKubelka 11/17/2015 13:14'!
printOn: aStream
	super printOn: aStream.
	aStream
		nextPut: $(;
		nextPutAll: (self isLiberated ifTrue: [ 'liberated' ] ifFalse: [ 'used' ]);
		nextPut: $)! !

Object subclass: #GTEventTool
	instanceVariableNames: 'semaphore process'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventTool commentStamp: 'JurajKubelka 11/15/2015 23:13' prior: 0!
I simplify the process of downloading and extracting of recorded data.

You likely want to use #ensureLocalDirectory that downloads and extracts server data or #unpackAll if you want to load all data into memory.!

!GTEventTool class methodsFor: 'accessing' stamp: 'JurajKubelka 11/15/2015 21:03'!
default
	^ self new! !

!GTEventTool methodsFor: 'private' stamp: 'JurajKubelka 11/15/2015 21:08'!
checkProcess
	process isComplete ifTrue: [ semaphore signal ].
! !

!GTEventTool methodsFor: 'configuration' stamp: 'JurajKubelka 11/15/2015 21:01'!
defaultDownloadDirectory
	^ FileSystem disk workingDirectory! !

!GTEventTool methodsFor: 'configuration' stamp: 'JurajKubelka 11/15/2015 21:01'!
defaultDownloadFilename
	^ self defaultDownloadDirectory / 'gt-events.tar.gz'! !

!GTEventTool methodsFor: 'configuration' stamp: 'JurajKubelka 11/15/2015 21:02'!
defaultEventsDirectory
	^ self defaultDownloadDirectory / 'gt' / 'events'! !

!GTEventTool methodsFor: 'private' stamp: 'JurajKubelka 11/15/2015 21:01'!
defaultUrl
	^ 'http://gc.dcc.uchile.cl/gt-events.tar.gz'! !

!GTEventTool methodsFor: 'api' stamp: 'JurajKubelka 11/15/2015 21:02'!
download
	"self default download"
 	UIManager default informUserDuring: [ :bar |
		bar label: 'Downloading Spotter Event Data...'.
		[ ZnClient new
			url: self defaultUrl;
			signalProgress: true;
			downloadTo: self defaultDownloadFilename ]
		on: HTTPProgress 
		do: [ :progress |
			progress isEmpty ifFalse: [ 
				bar current: progress percentage.
				progress total ifNotNil: [ :aTotalNumber | | humanReadable |
					humanReadable := self printHumanReadableSize: aTotalNumber.
					bar label: 'Downloading ', humanReadable, ' of Spotter Event Data... ' ] ].
			progress resume ] 
	]! !

!GTEventTool methodsFor: 'api' stamp: 'JurajKubelka 11/15/2015 21:05'!
ensureDownloadedData
	self defaultDownloadFilename exists ifFalse: [ self download ]! !

!GTEventTool methodsFor: 'api' stamp: 'JurajKubelka 11/15/2015 21:06'!
ensureExctractedData
	self defaultEventsDirectory exists ifFalse: [ self extract ]! !

!GTEventTool methodsFor: 'api' stamp: 'JurajKubelka 11/15/2015 21:09'!
ensureLocalDirectory
	"self default ensureLocalDirectory"
	self 
		ensureDownloadedData;
		ensureExctractedData! !

!GTEventTool methodsFor: 'api' stamp: 'AndreiChis 11/19/2015 20:40'!
extract
	semaphore := Semaphore new.
	(Smalltalk hasClassNamed: #ExternalUnixOSProcess) ifFalse: [ ^ self ].
	process := #ExternalUnixOSProcess asClass forkAndExec: '/usr/bin/tar' 
			arguments: (Array with: 'xzf' with: self defaultDownloadFilename fullName)
			environment: nil 
			descriptors: nil 
			workingDir: self defaultDownloadDirectory fullName.
	process addDependent: self.
	self checkProcess.
	semaphore wait.
	process := nil.
	semaphore := nil.! !

!GTEventTool methodsFor: 'printing' stamp: 'JurajKubelka 11/15/2015 21:03'!
printHumanReadableSize: aTotalNumber
	| humanReadable length unit |
	length := (aTotalNumber decimalDigitLength / 3) truncated.
	humanReadable := (aTotalNumber / (1024 raisedTo: (length min: 3))) rounded.
	length = 0 ifTrue: [ unit := 'bytes' ].
	length = 1 ifTrue: [ unit := 'KB' ].
	length = 2 ifTrue: [ unit := 'MB' ].
	length = 3 ifTrue: [ unit := 'GB' ].
	^ humanReadable printString, ' ', unit! !

!GTEventTool methodsFor: 'api' stamp: 'JurajKubelka 11/15/2015 22:39'!
unpackAll
	"self default unpackAll inspect"
	self ensureLocalDirectory.
	^ GTEventUnpacking default safeUnpackDirectory: self defaultEventsDirectory.! !

!GTEventTool methodsFor: 'api' stamp: 'AlexandreBergel 11/17/2015 12:02'!
unpackAll: aFileReference
	"self default unpackAll: './gt/events' asFileReference.
	self default unpackAll: './gt/events/roassal2Examples' asFileReference.
	" 
	self ensureLocalDirectory.
	^ GTEventUnpacking default safeUnpackDirectory: aFileReference.! !

!GTEventTool methodsFor: 'updating' stamp: 'JurajKubelka 11/15/2015 21:08'!
update: aParameter
	aParameter = #runState ifTrue: [ self checkProcess ]! !

Object subclass: #GTEventUnpacking
	instanceVariableNames: 'packings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Core'!

!GTEventUnpacking commentStamp: 'JurajKubelka 10/30/2015 12:28' prior: 0!
I represent an unpacking object that is able to receive serialized objects, e.g., FUEL or STON, and tries all available packing systems to unpack the serialized objects.

Others can ask me for materializing an object by sending me #unpack:. I resent the request to GTEventPacking objects and if anyone is able to materialize it, I will return the materialized object. Otherwise, I raise an error.

Collaborators Part: GTEventPacking, GTEventInitializationError, GTEventUnpackingError.

Public API and Key Messages

- unpack: aByteArray
- addPacking: aGTEventPacking
- how to create instances: #default on the class side.

Example:
	(GTEventUnpacking default
		unpack: (GTEventPacking ston 
					pack: (GTEventCollector new 
							add: 1; add: 2; add: 3; yourself)) 
			data) unpackedData
 
Internal Representation and Key Implementation Points.

    Instance Variables
	packings:		<OrderedCollection>
!

!GTEventUnpacking class methodsFor: 'instance creation' stamp: 'JurajKubelka 11/15/2015 22:43'!
default
	^ self new
			addPacking: GTEventPacking fuel;
			addPacking: GTEventPacking ston;
			yourself! !

!GTEventUnpacking methodsFor: 'adding' stamp: 'JurajKubelka 10/30/2015 12:03'!
addPacking: aGTEventPacking 
	^ self packings addLast: aGTEventPacking! !

!GTEventUnpacking methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 12:03'!
packings
	^ packings ifNil: [ packings := OrderedCollection new ]! !

!GTEventUnpacking methodsFor: 'packing' stamp: 'JurajKubelka 11/13/2015 16:18'!
safeUnpackDirectory: aFileReference
	"self default safeUnpackDirectory: './gt/events/' asFileReference"
	| unpackedData exceptions |
	exceptions := OrderedCollection new.
	[ unpackedData := self unpackDirectory: aFileReference ] on: GTEventUnpackingError do: [ :exception |
		exceptions addLast: exception.
		exception resume ].
	^ unpackedData -> exceptions! !

!GTEventUnpacking methodsFor: 'packing' stamp: 'JurajKubelka 11/2/2015 09:45'!
unpack: aByteArray
	| exceptions |
	exceptions := OrderedCollection new: self packings size.
	self packings do: [ :eachEventPacking |
		[ ^ eachEventPacking unpack: aByteArray ] on: Error
			do: [ :exception | exceptions addLast: exception ] ].
	exceptions
		ifEmpty: [ GTEventInitializationError signal: 'Uninitialized unpacking system' ]
		ifNotEmpty: [ GTEventUnpackingError 
								signal: 'Cannot unpack data' 
								data: aByteArray 
								errors: exceptions ].
	"In case of resumed exception it returns nil object."
	^ nil! !

!GTEventUnpacking methodsFor: 'packing' stamp: 'JurajKubelka 10/31/2015 18:50'!
unpackDirectory: aFileReference
	"self default unpackDirectory: './gt/events/' asFileReference"
	^ aFileReference asFileReference allFiles 
			collect: [ :eachFile | 
				(eachFile basename = '.DS_Store') 
					ifTrue: [ nil ]
					ifFalse: [ self unpackFile: eachFile ] ]
			thenSelect: #notNil! !

!GTEventUnpacking methodsFor: 'packing' stamp: 'JurajKubelka 11/1/2015 12:41'!
unpackFile: aFileReference
	"self default unpackFile: './gt/events/' asFileReference allFiles atRandom"
	^ [ self unpack: (aFileReference binaryReadStreamDo: [ :aStream |  aStream contents ]) ]
		on: GTEventUnpackingError do: [ :anException |
			anException data: aFileReference.
			anException pass ]! !

Object subclass: #GlobalIdentifier
	instanceVariableNames: 'persistenceStrategy persistedInformation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Identifier'!

GlobalIdentifier class
	instanceVariableNames: 'uniqueInstance'!

!GlobalIdentifier commentStamp: 'AndreiChis 10/12/2015 23:13' prior: 0!
I keep global IDs that are used for tracking user activity, e.g. computer id. By default, you should access me by calling #uniqueInstance.

Responsibility:
The IDs that are important to store onto disk (and shared by all images) should be placed in persistedInformation instance variable. On other hand, if you do not want to store it onto disk, create a new instance variable. I can #loadPreferences and #savePreferences onto a disk.

I know computer ID and secret ID. Computer ID is a global UUID that is share among all the images. It is stored on a local disk. Secret ID is use for encrypting information, e.g., class names, method names. You can use #hashForText: method.

Collaborators: I do not collaborate with other classes. I only offer the basic IDs for other frameworks.

Public API and Key Messages

- computerUUID 
- ensureComputerUUID
- hashForText:
- loadPreferences 
- savePreferences.

Before using #computerUUID, you should call #ensureComputerUUID. It will update UUID from the disk or stores existing one if it is not stored yet. I behave like this as automatic storing data to disk on image start-up leads to errors.

Internal Representation and Key Implementation Points.

    Instance Variables
	persistedInformation:		<Dictionary>
	preferences:		<FileReference>
!

GlobalIdentifier class
	instanceVariableNames: 'uniqueInstance'!

!GlobalIdentifier class methodsFor: 'cleanup' stamp: 'JurajKubelka 8/27/2015 10:44'!
cleanUp
	"Clean unique instance."	
	self reset.! !

!GlobalIdentifier class methodsFor: 'initialization' stamp: 'JurajKubelka 10/11/2015 16:28'!
initializeUniqueInstance
	uniqueInstance := self basicNew initialize.
	uniqueInstance ensure.! !

!GlobalIdentifier class methodsFor: 'instance creation' stamp: 'AndreiChis 9/15/2015 11:48'!
new
	"Users should not create instances of this class.
	Within the framework more instances are created when loading data and when testing."
	self shouldNotImplement! !

!GlobalIdentifier class methodsFor: 'initialize-release' stamp: 'JurajKubelka 8/27/2015 10:44'!
reset
	"self reset"
	uniqueInstance := nil! !

!GlobalIdentifier class methodsFor: 'accessing' stamp: 'JurajKubelka 10/11/2015 16:27'!
uniqueInstance
	"self uniqueInstance"
	uniqueInstance ifNil: [ self initializeUniqueInstance ].
	^ uniqueInstance! !

!GlobalIdentifier methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 21:23'!
computerUUID
	"This value identifies a user logged into the computer.
	Before calling #computerUUID method, you should call #ensure."
	^ persistedInformation at: #computerUUID! !

!GlobalIdentifier methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 21:32'!
ensure
	self persistenceStrategy ensure: self persistedInformation! !

!GlobalIdentifier methodsFor: 'initialization' stamp: 'JurajKubelka 10/10/2015 21:47'!
initialize
	super initialize.
	"Dictionary contains all the values that we want to persist on disk."
	persistedInformation := Dictionary new.
	persistedInformation at: #computerUUID put: UUID new asString.
	persistedInformation at: #secretUUID put: UUID new asString.
	persistenceStrategy := GlobalIdentifierPersistence ston.
	persistenceStrategy previous: GlobalIdentifierPersistence fuel.! !

!GlobalIdentifier methodsFor: 'actions' stamp: 'JurajKubelka 10/10/2015 20:37'!
obfuscate: anObject
	"If you have an information that you want to obfuscate before sending,
	this is likely a good solution for you."
	^ anObject hash bitXor: self secretUUID hash! !

!GlobalIdentifier methodsFor: 'actions' stamp: 'JurajKubelka 11/15/2015 20:17'!
obfuscateIfNecessary: anObject
	^ Privacy sendEntityNames
			ifTrue: [ anObject ]
			ifFalse: [ self obfuscate: anObject ]! !

!GlobalIdentifier methodsFor: 'accessing' stamp: 'JurajKubelka 9/2/2015 12:14'!
persistedInformation
	^ persistedInformation! !

!GlobalIdentifier methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 16:52'!
persistenceStrategy
	^ persistenceStrategy! !

!GlobalIdentifier methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 16:52'!
persistenceStrategy: anObject
	persistenceStrategy := anObject! !

!GlobalIdentifier methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 21:23'!
secretUUID
	"This value can be used for obfuscating an information before sending it.
	See #obfuscate: method.
	Before calling #secretUUID method, you should call #ensure."
	^ persistedInformation at: #secretUUID! !

Object subclass: #GlobalIdentifierMerger
	instanceVariableNames: 'existing stored'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Identifier'!

!GlobalIdentifierMerger commentStamp: 'JurajKubelka 10/30/2015 16:38' prior: 0!
I am able to merge dictionaries of existing (in the image) and stored (on a disk) values.

First, I identify if the image values belongs to the current computer. Then I decide to favor existing or stored values. It is importatnt to notice that I change the existing dictionary received by #existing: message. GlobalIdentifierPersistence used me when he needs to load values from the local disk.

Collaborators: GlobalIdentifierPersistence 

Public API and Key Messages

- existing:
- stored: 
- merge

Example:
	GlobalIdentifierMerger new
		existing: (Dictionary newFromPairs: #(#a 1 #b  2));
		stored: (Dictionary newFromPairs: #(#a 3 #c  4));
		merge;
		existing
 
Internal Representation and Key Implementation Points.

    Instance Variables
	existing:		<Dictionary>
	stored:		<Dictionary>
!

!GlobalIdentifierMerger methodsFor: 'actions' stamp: 'JurajKubelka 10/10/2015 22:12'!
convertValues
	existing 
		at: #computerUUID 
		ifPresent: [ :aValue | existing at: #computerUUID put: aValue asString ]
		ifAbsent: [ "ignore" ].
	existing 
		at: #secretUUID
		ifPresent: [ :aValue | existing at: #secretUUID put: aValue asString ]
		ifAbsent: [ "ignore" ].! !

!GlobalIdentifierMerger methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 19:31'!
existing
	^ existing! !

!GlobalIdentifierMerger methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 19:31'!
existing: anObject
	existing := anObject! !

!GlobalIdentifierMerger methodsFor: 'actions' stamp: 'JurajKubelka 10/10/2015 20:13'!
merge
	"If UUIDs are different, it is likely different user and we should use
	localy stored settings. If the UUIDs are same, we keep settings of the image.
	
	It returns true if existing dictionary is changed."
	^ ((existing at: #computerUUID ifAbsent: nil) = (stored at: #computerUUID ifAbsent: nil)
		or: [ (stored includesKey: #computerUUID) not ])
			ifTrue: [ self mergeSameComputer ]
			ifFalse: [ self mergeDifferentComputer ]! !

!GlobalIdentifierMerger methodsFor: 'actions' stamp: 'JurajKubelka 10/10/2015 22:07'!
mergeDifferentComputer
	"It puts all stored values into existing dictionary."
	stored keysAndValuesDo: [ :eachKey :eachValue |
		existing at: eachKey put: eachValue ].
	self convertValues.
	^ true! !

!GlobalIdentifierMerger methodsFor: 'actions' stamp: 'JurajKubelka 10/10/2015 22:13'!
mergeSameComputer
	"It add into existing dictionary keys that are included in the stored dictionary,
	but are missing in the existing dictionary. Useful when a new version
	adds new configuration values that should be saved too.
	
	It returns true if existing dictionary is changed."
	| difference |
	difference := stored keys difference: existing keys.
	difference ifEmpty: [ ^ false ].
	difference do: [ :eachKey |
		existing at: eachKey put: (stored at: eachKey) ].
	self convertValues.
	^ true! !

!GlobalIdentifierMerger methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 19:31'!
stored
	^ stored! !

!GlobalIdentifierMerger methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 19:31'!
stored: anObject
	stored := anObject! !

Object subclass: #GlobalIdentifierPersistence
	instanceVariableNames: 'preferences previousPersistence'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Identifier'!

!GlobalIdentifierPersistence commentStamp: 'JurajKubelka 10/30/2015 16:53' prior: 0!
I represent persistence strategy for GlobalIdentifier. 

I can load or save existing preferences (a dictionary) using #load: or #save: message. I know whare the preference file is stored (#preferences). I also know what previous persistence stategy was used; for that reason you should use #ensure: instead of #load: message. In that case, I will load the old file and then save it using new persistence strategy.

Collaborators: GlobalIdentifier uses me for loading and saving values; in both cases GlobalIdentifier uses #ensure:

Public API and Key Messages

- ensure: it loads stored values, merge with existing in the image and saves the current state on the disk

Example:
	GlobalIdentifierPersistence ston
		previous: (GlobalIdentifierPersistence fuel
					preferences: FileLocator workingDirectory / 'example.fuel';
					yourself);
		preferences: FileLocator workingDirectory / 'example.ston';
		ensure: (Dictionary new at: #a put: 1; at: #b put: 2);
		yourself

Internal Representation and Key Implementation Points.

    Instance Variables
	preferences:				<FileReference>
	previousPersistence:		<GlobalIdentifierPersistence>
!

GlobalIdentifierPersistence subclass: #GlobalIdentifierFuelPersistence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Identifier'!

!GlobalIdentifierFuelPersistence commentStamp: 'JurajKubelka 10/30/2015 16:55' prior: 0!
I serialize or materialize objects using FUEL framework. Create instance of me calling GlobalIdentifierPersistence fuel.!

!GlobalIdentifierFuelPersistence methodsFor: 'configuration' stamp: 'JurajKubelka 10/10/2015 21:07'!
fileName 
	^ 'org.pharo.gt.spotter.event.recorder.fuel'! !

!GlobalIdentifierFuelPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 20:17'!
load
	^ FLMaterializer materializeFromFileNamed: self preferences! !

!GlobalIdentifierFuelPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 20:31'!
save: aDictionary into: aWriteStream
	aWriteStream binary.
	[ FLSerializer serialize: aDictionary on: aWriteStream ]
		on: Error
		do: [ :anError | "ignore" ]! !

!GlobalIdentifierPersistence class methodsFor: 'instance creation' stamp: 'JurajKubelka 10/10/2015 18:46'!
fuel
	^ GlobalIdentifierFuelPersistence new! !

!GlobalIdentifierPersistence class methodsFor: 'instance creation' stamp: 'JurajKubelka 10/10/2015 20:58'!
ston
	^ GlobalIdentifierStonPersistence new! !

!GlobalIdentifierPersistence methodsFor: 'configuration' stamp: 'JurajKubelka 10/10/2015 21:08'!
defaultPreferences
	^ FileLocator preferences / self fileName! !

!GlobalIdentifierPersistence methodsFor: 'operations' stamp: 'JurajKubelka 10/10/2015 19:13'!
delete
	"self deletePreferences"
	[ self preferences ensureDelete ]
	on: Error do: [ "ignore" ]! !

!GlobalIdentifierPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 21:42'!
ensure: existingDictionary
	"It proceeds all backward compatibility work."
	self shouldCallPreviousPersistence ifTrue: [ 
		previousPersistence ensure: existingDictionary ].
	self load: existingDictionary! !

!GlobalIdentifierPersistence methodsFor: 'operations' stamp: 'JurajKubelka 10/10/2015 19:14'!
ensureDirectory
	self preferences parent ensureCreateDirectory ! !

!GlobalIdentifierPersistence methodsFor: 'configuration' stamp: 'JurajKubelka 10/10/2015 21:10'!
fileName
	^ self subclassResponsibility! !

!GlobalIdentifierPersistence methodsFor: 'testing' stamp: 'JurajKubelka 11/16/2015 11:38'!
hasPreviousPersistence
	^ previousPersistence notNil! !

!GlobalIdentifierPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 20:18'!
load
	"It loads and returns stored dictionary."
	^ self subclassResponsibility! !

!GlobalIdentifierPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 11/16/2015 11:36'!
load: existingDictionary
	"It loads stored information into existingDictionary."
	self preferences exists ifFalse: [
		"This is a new computer, so we define new computer UUID.
		User still has to agree about sending data if it is not has been done yet."
		^ self save: existingDictionary ].
	[ (self mergeExisting: existingDictionary stored: self load)
			ifTrue: [ self save: existingDictionary ].
	] on: Error do: [ 
		"Preferences likely contains a different settings version, so we store the actual one.
		We should keep the preferences as stable as possible."
		self mayOverwrite: existingDictionary ]! !

!GlobalIdentifierPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 11/16/2015 11:40'!
mayOverwrite: aDictionary
	"If I keep previous persistence strategy, I am the main one and I can overwrite the current preference file.
	I am called only on loading failure; it means when I cannot load preference file."
	self hasPreviousPersistence ifTrue: [ self save: aDictionary ].! !

!GlobalIdentifierPersistence methodsFor: 'merging' stamp: 'JurajKubelka 10/10/2015 20:17'!
mergeExisting: existingDictionary stored: storedDictionary
	^ GlobalIdentifierMerger new
			existing: existingDictionary;
			stored: storedDictionary;
			merge! !

!GlobalIdentifierPersistence methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 21:08'!
preferences
	^ preferences ifNil: [ preferences := self defaultPreferences ]! !

!GlobalIdentifierPersistence methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 16:39'!
preferences: anObject
	preferences := anObject! !

!GlobalIdentifierPersistence methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 21:45'!
previous
	^ previousPersistence! !

!GlobalIdentifierPersistence methodsFor: 'accessing' stamp: 'JurajKubelka 10/10/2015 21:45'!
previous: aGlobalIdentifierPersistence
	previousPersistence := aGlobalIdentifierPersistence! !

!GlobalIdentifierPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 20:29'!
save: aDictionary
	self delete.
	self ensureDirectory.
	self preferences writeStreamDo: [ :aPreferenceStream | 
			self save: aDictionary into: aPreferenceStream ]! !

!GlobalIdentifierPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 20:32'!
save: aDictionary into: aWriteStream
	"Saves aDictionary into aWriteStream"
	self subclassResponsibility! !

!GlobalIdentifierPersistence methodsFor: 'testing' stamp: 'JurajKubelka 11/16/2015 11:37'!
shouldCallPreviousPersistence
	^ self preferences exists not and: [ self hasPreviousPersistence ]! !

GlobalIdentifierPersistence subclass: #GlobalIdentifierStonPersistence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Identifier'!

!GlobalIdentifierStonPersistence commentStamp: 'JurajKubelka 10/30/2015 16:55' prior: 0!
I serialize or materialize objects using STON framework. Create instance of me calling GlobalIdentifierPersistence ston.!

!GlobalIdentifierStonPersistence methodsFor: 'configuration' stamp: 'JurajKubelka 10/10/2015 21:07'!
fileName
	^ 'org.pharo.global-identifiers.ston'! !

!GlobalIdentifierStonPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 21:15'!
load
	^ self preferences readStreamDo: [ :aReadStream |
			STON fromStream: aReadStream ]! !

!GlobalIdentifierStonPersistence methodsFor: 'load and saving' stamp: 'JurajKubelka 10/10/2015 21:02'!
save: aDictionary into: aWriteStream
	STON put: aDictionary onStreamPretty: aWriteStream! !

Object subclass: #Privacy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-EventRecorder-Privacy'!

Privacy class
	instanceVariableNames: 'sendDiagnosticsAndUsageData sendEntityNames sendSourceCode'!

!Privacy commentStamp: 'JurajKubelka 9/2/2015 12:56' prior: 0!
I represent basic privacy question about collecting user activities and other data that helps to improve Pharo and related tools.

Responsibility: 
I keep information about sending diagnostic and usage data. 

Collaborators:
I only keep the privacy information. All services that collects diagnostic and usage data should do it only if #sendDiagnosticsAndUsageData is true. 

If you want to add more specific settings related to privacy, you can put it to the privacy group.

Public API and Key Messages

- sendDiagnosticsAndUsageData   
!

Privacy class
	instanceVariableNames: 'sendDiagnosticsAndUsageData sendEntityNames sendSourceCode'!

!Privacy class methodsFor: 'cleanup' stamp: 'JurajKubelka 10/30/2015 15:16'!
cleanUp
	"Clean settings."
	sendDiagnosticsAndUsageData := nil.
	sendEntityNames := nil.
	sendSourceCode := nil.! !

!Privacy class methodsFor: 'settings' stamp: 'JurajKubelka 6/18/2015 17:05'!
privacySettingsOn: aBuilder
	<systemsettings>
	(aBuilder group: #privacy)
		label: 'Privacy';
		noOrdering;
		description: 'All settings concerned with privacy'! !

!Privacy class methodsFor: 'accessing' stamp: 'AndreiChis 11/9/2015 15:33'!
sendDiagnosticsAndUsageData
	^ sendDiagnosticsAndUsageData ifNil: [ sendDiagnosticsAndUsageData := false ]! !

!Privacy class methodsFor: 'accessing' stamp: 'JurajKubelka 12/18/2015 12:12'!
sendDiagnosticsAndUsageData: aBoolean
	sendDiagnosticsAndUsageData = aBoolean ifTrue: [ ^ self ].
	sendDiagnosticsAndUsageData := aBoolean.
	SystemAnnouncer uniqueInstance announce: PrivacyChangedAnnoucement.! !

!Privacy class methodsFor: 'settings' stamp: 'JurajKubelka 11/13/2015 11:20'!
sendDiagnosticsAndUsageUsageDataSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #sendDiagnosticsAndUsageData)
		label: 'Send diagnostics and usage data';
		noOrdering;
		description: 'Help the Pharo community improve its products and services by automatically sending diagnostic and usage data.';
		default: false;
		parent: #privacy! !

!Privacy class methodsFor: 'accessing' stamp: 'AndreiChis 11/9/2015 15:33'!
sendEntityNames
	^ sendEntityNames ifNil: [ sendEntityNames := false ]! !

!Privacy class methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 15:16'!
sendEntityNames: aBoolean
	sendEntityNames := aBoolean! !

!Privacy class methodsFor: 'settings' stamp: 'JurajKubelka 11/13/2015 11:20'!
sendEntityNamesSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #sendEntityNames)
		label: 'Include package/class/method names';
		noOrdering;
		description: 'Help the Pharo community improve its products and services by automatically sending package, class, or method names as part of the diagnostic and usage data, e.g., browsed classes, changed methods.';
		default: false;
		parent: #sendDiagnosticsAndUsageData! !

!Privacy class methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 15:16'!
sendSourceCode
	^ sendSourceCode ifNil: [ sendSourceCode := false ]! !

!Privacy class methodsFor: 'accessing' stamp: 'JurajKubelka 10/30/2015 15:16'!
sendSourceCode: aBoolean
	sendSourceCode := aBoolean! !

!Privacy class methodsFor: 'settings' stamp: 'AndreiChis 11/8/2015 20:14'!
sendSourceCodeSettingsOn: aBuilder
	<systemsettings>
	(aBuilder setting: #sendSourceCode)
		label: 'Include source code';
		noOrdering;
		description: 'Help the Pharo community improve its products and services by automatically sending source code as part of the diagnostic and usage data, e.g., changed methods, browsed source code, written code snippets.';
		default: false;
		parent: #sendDiagnosticsAndUsageData! !
